% Code to solve the PHEV design and allocation problems
%   by CS Norman Shiau cshiau@cmu.edu
% Carnegie Mellon University, Pittsburgh, PA
%
% ----
% Update by CS Norman Shiau 4/10/2010 
%  (1) updated the exp distribution coefficient to NHTS 2009 weighted data
%  (2) added battery charging efficient effBAT (95%)
%  (3) Updated US avg. elec. emission factor to 0.69/kWh 
%
% ----
% Updated by CS Norman Shiau 6/5/2020
% Code to run the 5 segment-combinations of 3-vehicle models
% Combined codes to solve 3-vehicle cases automatically
%   CV  -HEV
%   CV  -PHEV
%   HEV -PHEV
%   PHEV-HEV
%   PHEV-PHEV
%
% ----
% Updated 6/7/2010
%  Added BATCON option for battery life constraint
%  Added BATCRD option for battery tax credit

format compact
format long
clear all
close all
clc
%diary output_cost_2veh_test.txt

global PTCM BATDR BATAS BATEOL VEHLIFE BATCON BATCRD options RS
global ss k dmax prob D rR rI rN 
global v c sLIFE yLIFE HEV CV alpha kappa effCHG effBAT ctax

ctax = 0.0; % carbon tax $/kg-CO2-eq (e.g. 0.1 = $100/tonne)

% Flag for battery degradation model
BATDR='P'; % Use Peterson energy-based degradation curve
% BATDR='R'; % Use Rosenkranz DoD-based degradation curve

% Flag for battery assignment scenario
BATAS='1'; % each vehicle has at least 1 battery (buy-lease)
% BATAS='L'; % battery leasing

% Flag for powertrain cost model
PTCM='E'; % EPRI cost model (base case)
% PTCM='L'; % the LR model in the 2009 ANL report
% PTCM='P'; % the PG model in the 2009 ANL report

% Flag for battery EOL criterion
BATEOL='S'; % rEOL = 1-swing
% BATEOL='2'; % rEOL = fixed at 20%

% Flag for vehicle life assumption
VEHLIFE='M'; % constant vehicle life VMT = 150,000 miles
% VEHLIFE='Y'; % constant vehicle life years = 12 years

% Flag for battery life constraint
BATCON='N'; % no battery constraint
% BATCON='Y'; % enable battery life constraint to outlast vehicle life

% Flag for the tax credit in the AARA bill (Obama's stimulus package)
BATCRD='N'; % no tax credit
% BATCRD='Y'; % include tax credit

rN=0.05; % Nominal discount rate
rI=0.03; % 2008 annual average inflation rate = 3.85%
% http://inflationdata.com/inflation/Inflation_Rate/CurrentInflation.asp
rR=(rN+1)/(rI+1)-1; % real discount rate
D=300; % 300 driving days per year
%
dmax=200;    % Maximum range in driving data (200 miles)
sLIFE=150e3; % vehicle life VMT mile (for constant VMT life assumption)
yLIFE=12;    % vehicle life years (for constant life year assumption)
ss=1e-2;
dd=0:ss:dmax;
k=0.0296;  % Exp dist coefficient on the NHTS 2009 weighted data
FS=@(y)1-exp(-k*y);
FE=@(y)1/k-exp(-k*y)*(y+1/k);
prob=exppdf(dd,1/k);
%
c.gas   = 3.30;  % Gas price = $3.30/gal (high:$6.0, low:$1.5)
c.elec  = 0.11;  % Retail electricity price = $0.11/kWh (high:0.30, low:0.06)
c.LiIon = 400;   % Li-ion battery cost = 400 (high: 1000, low: 250)
c.NiMH  = 600;   % NiMH   battery cost = 600 (high: 700,  low: 440)
c.BASE  = 11183; % vehicle base cost (high: 13000, low: 9000)
%
v.gas   = 11.34; % LC-GHGs of gasoline per gallon = 11.34 kg/gal
v.elec  = 0.752; % LC-GHGs of US avg. = 0.69*1.09 = 0.752 kg/kWh
% v.elec  = 0.072; % LC-GHGs of nuclear = 0.066*1.09 = 0.072 kg/kWh
% v.elec  = 0.275; % 100% ICCG with CCS = 0.275 kgCO2e/kWh
% v.elec  = 0.510; % 100% natural gas = 0.468*1.09 = 0.51 kgCO2e/kWh
% v.elec  = 0.981; % 100% coal = 0.9*1.09 = 0.981 kgCO2e/kWh
v.BASE  = 8500;  % LC-GHGs of base vehicle manufacturing
v.NiMH  = 230;   % LC-GHGs of NiMH   battery manufacturing 230 kg CO2-eq/kWh
v.LiIon = 120;   % LC-GHGs of li-ion battery manufacturing 120 kg CO2-eq/kWh
%
effCHG = 0.88;   % charging efficiency between plug and battery
effBAT = 0.95;   % charging efficiency in battery
% battery cell nominal voltage = 3.6 volt
% battery cell capacity = 6 Ah
% energy capacity = 6*3.6 = 21.6 Wh/cell = 21.6e-3 kWh/cell
kappa=21.6e-3; % battery cell energy capacity in kWh 
alpha.DRV=3.458e-5; % REC slope on driving  in Peterson model (Wh/Wh)
alpha.CHG=1.720e-5; % REC slope on charging in Peterson model (Wh/Wh)
%
RS=100; % number of random multi-starts
options = optimset('LargeScale','off','Display','off','FinDiffType','central', ...
        'Algorithm','Active-set', ...
        'MaxFunEvals',1e6,'TolX',1e-6,'TolFun',1e-6,'TolCon',1e-6);
%
[HEV,CV]=getHEVCV(dd);
%
disp('------- 2 VEHICLE-SEGMENT MODEL ------');
%
if VEHLIFE=='M',
    disp('Vehicle life assumption: constant life VMT 150k miles');
elseif VEHLIFE=='Y',
    disp('Vehicle life assumption: constant life 12 years');
else
    disp('VEHLIFE flag error.');  break;
end
%
if BATEOL=='S',
    disp('Battery end-of-life criterion: rEOL = 1-swing');
elseif BATEOL=='2',
    disp('Fixed battery EOL at 20% capacity fade');
else
    disp('BATEOL flag error.');  break;
end
%
if BATDR=='P',
    disp('Using Peterson energy-based degradation model');
elseif BATDR=='R',
    disp('Using Rosenkranz DOD-based degradation model');
else
    disp('BATDR flag error.');  break;
end
%
if BATAS=='L', 
    disp('Battery scenario: Leasing');
elseif BATAS=='1',
    disp('Battery scenario: Buy-lease (at least 1 battery)');
else
    disp('BATAS flag error.');  break;
end
%
if PTCM=='E',
    disp('Powertrain cost is calculated using EPRI cost model');
elseif PTCM=='L';
    disp('Powertrain cost is calculated using ANL-LR model');
elseif PTCM=='P';
    disp('Powertrain cost is calculated using ANL-PG model');
else
    disp('PTCM flag error.');  break;
end
%
if BATCON=='Y',
    disp('Battery life constraint (>= Vehicle life) is enabled');
elseif BATCON=='N',
    disp('Battery life constraint (>= Vehicle life) is NOT enabled.');
else
    disp('BATCON flag error.'); break;
end
%
if BATCRD=='Y',
    disp('ARRA tax credit for battery is included');
elseif BATCRD=='N', % no tax credit
    disp('No tax credit for battery');
else
    disp('BATCRD flag error.'); break;
end
%
disp('---');
disp(sprintf(' Li-ion battery cost = %0.0f $/kWh',c.LiIon ));
disp(sprintf('   NiMH battery cost = %0.0f $/kWh',c.NiMH ));
disp(sprintf('      Gasoline price = %0.2f $/gal',c.gas ));
disp(sprintf('   Electricity price = %0.2f $/kWh',c.elec ));
disp(sprintf('Grid emission factor = %0.3f kgCO2e/kWh',v.elec ));
disp(sprintf('           carbon tax = %0.0f $/ton',ctax*1000'));
disp(sprintf('Nominal discount rate = %-0.2f%%',rN*100 ));
disp(sprintf('   Real discount rate = %-0.2f%%',rR*100 ));
disp(sprintf('       Inflation rate = %-0.2f%%',rI*100 ));
disp('--------------------------------------');
%
disp('Minimizing EAC per day');
disp('-----------------------------------------');
% CV -HEV  (CH)
[mincostCH,xdCH]=runcostCH;
dispCH(xdCH);
disp(xdCH)
disp('------------------------------------------');
% CV -PHEV (CP)
[mincostCP,xdCP]=runcostCP;
dispCP(xdCP);
disp(xdCP)
disp('------------------------------------------');
% HEV -PHEV (HP)
[mincostHP,xdHP]=runcostHP;
dispHP(xdHP);
disp(xdHP)
disp('------------------------------------------');
% PHEV-HEV  (PH)
[mincostPH,xdPH]=runcostPH;
dispPH(xdPH);
disp(xdPH)
disp('------------------------------------------');
% PHEV-PHEV  (PP)
[mincostPP,xdPP]=runcostPP;
dispPP(xdPP);
disp(xdPP)
disp('------------------------------------------');
%
disp('---- summary ----');
disp(sprintf('Min-cost for   CV- HEV = %0.6f',mincostCH));
disp(sprintf('Min-cost for   CV-PHEV = %0.6f',mincostCP));
disp(sprintf('Min-cost for  HEV-PHEV = %0.6f',mincostHP));
disp(sprintf('Min-cost for PHEV- HEV = %0.6f',mincostPH));
disp(sprintf('Min-cost for PHEV-PHEV = %0.6f',mincostPP));
%
%diary off
